//
// Copyright (c) 2016, MindFusion LLC - Bulgaria.
//

using System;
using System.IO;
using System.Drawing;
using System.Windows.Forms;


namespace MindFusion.Scheduling.WinForms.Samples.CS.CustomEvents
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		/// <summary>
		/// Initializes a new instance of the MainForm class.
		/// </summary>
		public MainForm()
		{
			InitializeComponent();

			_appPath = System.IO.Path.GetDirectoryName(Application.ExecutablePath);

			_lbInfo.DocumentText = Properties.Resources.Info;
			_lbInfo.DocumentCompleted += new WebBrowserDocumentCompletedEventHandler(OnDocumentCompleted);

			// To be able to serialize custom events,
			// we have to register their types in the
			// class table of the underlying schedule
			Schedule.RegisterItemClass(typeof(MyEvent), "customEvent", 1);

			// Make the control create events of our type
			// when the user types directly in the control
			_schedule.InteractiveItemType = typeof(MyEvent);

			// Set custom drawing for items
			_schedule.CustomDraw = CustomDrawElements.TimetableItem;

			// Set the displayed date to today
			DateTime today = DateTime.Today;
			_schedule.TimetableSettings.Dates.Clear();
			_schedule.TimetableSettings.Dates.Add(today);

			// Create one event initially and add it to the schedule
			MyEvent myEvent = new MyEvent();
			myEvent.HeaderText = "My Event";
			myEvent.DescriptionText = "This is the event's description.";
			myEvent.CustomField = "This is my event.\nCool, isn't it?";
			myEvent.StartTime = today + new TimeSpan(180 * TimeSpan.TicksPerMinute);
			myEvent.EndTime = today + new TimeSpan(240 * TimeSpan.TicksPerMinute);
			_schedule.Schedule.Items.Add(myEvent);

			_schedule.Theme = ThemeType.Light;
		}

		private void OnDocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
		{
			if (_lbInfo.Document != null)
			{
				_lbInfo.Document.Click += Document_Click;
				_lbInfo.Document.Body.Style = "font-family: Verdana, Arial, Helvetica, sans-serif; " +
					"font-size: 7.5pt; margin: 2pt; padding: 0pt; background-color: #ffffee; cursor: default;";
			}
		}

		/// <summary>
		/// Handles the Web browser document's Click event.
		/// </summary>
		private void Document_Click(object sender, HtmlElementEventArgs e)
		{
			e.ReturnValue = true;

			HtmlElement pointed = _lbInfo.Document.GetElementFromPoint(e.ClientMousePosition);
			if (pointed == null)
				return;

			if (pointed.TagName.ToLower() == "a")
			{
				string href = pointed.GetAttribute("href").ToLower();
				if (href.StartsWith("help:"))
				{
					string topic = href.Substring(5);
					string helpFile = ResolveHelpPath(_appPath);

					if (System.IO.File.Exists(helpFile))
					{
						Help.ShowHelp(this, helpFile, HelpNavigator.Topic, topic);

						// Prevent link follow
						e.ReturnValue = false;
					}
				}
			}
		}

		private static string ResolveHelpPath(string baseDir)
		{
			string[] helpFiles = new string[]
			{
				"MindFusion.Scheduling.chm",
				"MindFusion WinForms.chm"
			};

			System.IO.DirectoryInfo dirInfo = new System.IO.DirectoryInfo(baseDir);
			int i = 0;
			while (dirInfo != null)
			{
				string helpFile = System.IO.Path.Combine(dirInfo.FullName, helpFiles[i]);
				if (System.IO.File.Exists(helpFile))
					return helpFile;

				i++;
				if (i == helpFiles.Length)
				{
					i = 0;
					dirInfo = dirInfo.Parent;
				}
			}

			return string.Empty;
		}

		private void OnAbout(object sender, System.EventArgs e)
		{
			System.Reflection.Assembly assembly = System.Reflection.Assembly.GetAssembly(
				typeof(MindFusion.Scheduling.WinForms.Calendar));

			if (assembly == null)
				return;

			Type aboutFormType = assembly.GetType(
				"MindFusion.Scheduling.WinForms.AboutForm");

			if (aboutFormType == null)
				return;

			((Form)Activator.CreateInstance(aboutFormType)).ShowDialog(this);
		}

		private void OnExit(object sender, System.EventArgs e)
		{
			this.Close();
		}

		private void _menuSave_Click(object sender, System.EventArgs e)
		{
			if (_saveDialog.ShowDialog(this) == DialogResult.OK)
			{
				// Save to Xml for example
				_schedule.Schedule.SaveTo(_saveDialog.FileName, ContentType.Xml);
			}
		}

		private void _menuOpen_Click(object sender, System.EventArgs e)
		{
			if (_openDialog.ShowDialog(this) == DialogResult.OK)
			{
				// Load the schedule from a Xml
				try
				{
					_schedule.Schedule.LoadFrom(_openDialog.FileName, ContentType.Xml);
				}
				catch
				{
					MessageBox.Show("Invalid file format.");
				}
				_schedule.Invalidate();
			}
		}

		private void _schedule_ItemClick(object sender, MindFusion.Scheduling.WinForms.ItemMouseEventArgs e)
		{
			if (e.Button == MouseButtons.Left && e.Clicks > 1)
			{
				_schedule.ResetDrag();

				// Display the custom field of the clicked event
				MessageBox.Show(this, (e.Item as MyEvent).CustomField, "Custom Event");
			}
		}

		private void _schedule_Draw(object sender, MindFusion.Scheduling.WinForms.DrawEventArgs e)
		{
			if (e.Element == CustomDrawElements.TimetableItem)
			{
				MyEvent myEvent = _schedule.Schedule.Items[e.Index] as MyEvent;
				string customField = myEvent.CustomField;

				Font font = e.Style.Font;
				if (font == null)
					font = _schedule.ItemSettings.Style.Font;
				if (font == null)
					font = _schedule.TimetableSettings.Style.Font;

				SizeF size = e.Graphics.MeasureString(customField, font);

				if (size.Width > e.Bounds.Width - 4)
					size.Width = e.Bounds.Width - 4;
				if (size.Height > e.Bounds.Height - 4)
					size.Height = e.Bounds.Height - 4;

				// Draw the custom field in the lower right corner of the item
				Rectangle bounds = new Rectangle(
					e.Bounds.Right - (int)size.Width - 3,
					e.Bounds.Bottom - (int)size.Height - 3,
					(int)size.Width, (int)size.Height);

				e.Graphics.FillRectangle(Brushes.Yellow, bounds);
				e.Graphics.DrawRectangle(Pens.Black, bounds);
				e.Graphics.DrawString(customField, font, Brushes.Black, bounds.X, bounds.Y);
			}
		}


		private string _appPath;
	}


	/// <summary>
	/// This is our custom event. It has a single custom field.
	/// </summary>
	public class MyEvent : MindFusion.Scheduling.Appointment
	{
		/// <summary>
		/// Custom events are required to provide
		/// parameterless constructor.
		/// </summary>
		public MyEvent()
		{
			// Set the custom field to contain the
			// time when the event was created
			_customField = DateTime.Now.ToString("hh:mm tt");
		}


		/// <summary>
		/// A SaveTo override that provides custom serialization
		/// for our event.
		/// </summary>
		public override void SaveTo(BinaryWriter writer, BinarySerializationContext context)
		{
			context.WriteString(_customField, writer);
			base.SaveTo(writer, context);
		}

		/// <summary>
		/// A SaveTo override that provides custom deserialization
		/// for our event.
		/// </summary>
		public override void LoadFrom(BinaryReader reader, BinarySerializationContext context)
		{
			_customField = context.ReadString(reader);
			base.LoadFrom(reader, context);
		}

		/// <summary>
		/// A SaveTo override that provides custom serialization
		/// to XML documents for our event.
		/// </summary>
		public override void SaveTo(System.Xml.XmlElement element, XmlSerializationContext context)
		{
			context.WriteString(_customField, "customField", element);
			base.SaveTo(element, context);
		}

		/// <summary>
		/// A LoadFrom override that provides custom deserialization
		/// from XML document for our event.
		/// </summary>
		public override void LoadFrom(System.Xml.XmlElement element, XmlSerializationContext context)
		{
			_customField = context.ReadString("customField", element);
			base.LoadFrom(element, context);
		}

		/// <summary>
		/// A CopyOccurrence override that enables support of recurrence
		/// with our event.
		/// </summary>
		protected override void CopyOccurrence(Item master)
		{
			_customField = (master as MyEvent).CustomField;

			base.CopyOccurrence(master);
		}

		/// <summary>
		/// A Clone override that enables interactive item cloning.
		/// </summary>
		public override object Clone()
		{
			MyEvent clone = new MyEvent();

			// The following code replicates the code used in
			// the Appointment's Clone method
			clone.AllDayEvent = this.AllDayEvent;
			clone.DescriptionText = this.DescriptionText;
			clone.EndTime = this.EndTime;
			clone.HeaderText = this.HeaderText;
			clone.Location = this.Location;
			clone.Locked = this.Locked;
			clone.Priority = this.Priority;
			clone.Reminder = this.Reminder;
			clone.SelectedStyle = this.SelectedStyle.Clone() as Style;
			clone.StartTime = this.StartTime;
			clone.Style = this.Style.Clone() as Style;
			clone.Tag = this.Tag;
			clone.Task = this.Task;
			clone.Visible = this.Visible;

			foreach (Resource resource in this.Resources)
				clone.Resources.Add(resource);

			foreach (Contact contact in this.Contacts)
				clone.Contacts.Add(contact);

			// Now copy the custom fields
			clone.CustomField = this.CustomField;

			return clone;
		}


		/// <summary>
		/// Gets or sets the custom field.
		/// </summary>
		public string CustomField
		{
			get { return _customField; }
			set
			{
				_customField = value;

				// Mark this event as an exception if it is a recurrence instance
				if (Recurrence != null)
					Recurrence.MarkException(this, false);
			}
		}


		private string _customField;
	}
}
